/*** Program: 0a_claims_based_frailty_index.sas                                                                 ***/
/*** Program description: create frailty index                                                                  ***/
/*** Downloaded 8/13/2020 from: https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/HM8DOI ***/
/*** Modified to use with our data/sample with ICD-10 codes                                                     ***/
/*** Need: reference files downloaded from dataverse link above, claims files with ICD-10 codes and HCPCS codes ***/

libname denom 'D:\'; * location of sample denominator dataset;
libname claims 'D:\'; * input directories for claims files ;
libname r 'D:\'; * directory with frailty index reference files (downloaded from link above) ;
libname dataout 'D:\'; * output directory ;

%let pyr=2016; * Assign previous year - year all claims and indicators are based on ;
%let yr=2017;  * Assign current year - year the segments will be used               ;
%let denom_file=denom.denom_&pyr._&yr._2yr; *change this to denominator dataset in format lib.dataset (example: denom.denom_&pyr._&yr._2yr);
                                            *this should have id as bene_id, and an age variable for age in previous year as age_&pyr. ;


/*** Section 1. Prep claims data - Create files of codes in long file: bene_id, code ***/
/*** using all claim files from the year                                  ***/

*****************************************;
* create datasets:     variables:        ;
* 1.dataout.dx10_&pyr. bene_id, dx(icd10);
* 2.dataout.px_&pyr.   bene_id, px(CPT4) ;
*****************************************;

/* diagnosis codes */
data dataout.dx10_&pyr.;
	set claims.Inptclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Otptclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Bcarclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Hhaclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Hspcclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Snfclms&pyr.(keep=bene_id ICD_DGNS10_:)
		claims.Dmeclms&pyr.(keep=bene_id ICD_DGNS10_:);

	array diag {*} ICD_DGNS10_CD:;

	do i=1 to dim(diag);
		if diag[i] ne '' then do;
			dx=diag[i];
			output;
		end;
	end;

	keep bene_id dx;
run;

proc sort data=dataout.dx10_&pyr. nodupkey; by bene_id dx; run; /* to reduce file size - only need one instance of a code for each bene */

/* Use revenue/line files to get all unique hcpcs_cd/cpt from previous year for each bene */
%macro get_hcpcs(filen);
data px_&filen._&pyr.;
	set claims.&filen.&pyr.(keep=bene_id hcpcs_cd where=(hcpcs_cd ne ''));

	keep bene_id hcpcs_cd;
run;

proc sort data=px_&filen._&pyr. nodupkey; by bene_id hcpcs_cd; run;
%mend get_hcpcs;

%get_hcpcs(inptrev);
%get_hcpcs(Otptrev);
%get_hcpcs(Bcarline);
%get_hcpcs(Hharev);
%get_hcpcs(Hspcrev);
%get_hcpcs(Snfrev);
%get_hcpcs(Dmeline);

data dataout.px_&pyr.;
	set px_inptrev_&pyr.
		px_otptrev_&pyr.
		px_bcarline_&pyr.
		px_hharev_&pyr.
		px_hspcrev_&pyr.
		px_snfrev_&pyr.
		px_dmeline_&pyr.;
run;

proc sql; drop table px_inptrev_&pyr.; quit; proc sql; drop table px_otptrev_&pyr.; quit;
proc sql; drop table px_bcarline_&pyr.; quit; proc sql; drop table px_hharev_&pyr.; quit;
proc sql; drop table px_hspcrev_&pyr.; quit; proc sql; drop table px_snfrev_&pyr.; quit; proc sql; drop table px_dmeline_&pyr.; quit;

proc sort data=dataout.px_&pyr. nodupkey; by bene_id hcpcs_cd; run; /* only need one instance of a code for each bene */


/*** Section 2. Don't Edit - Taken from online code (just deleted ICD-9 part since we don't have any ICD-9 codes) ***/

**************************************;
* scores data:                        ;
* 1.codes_org                         ;
* 2.disease_weight                    ;
* 3.frailty_icd10_dx                  ;
**************************************;

data px_score dx_score;
	set r.codes_org;

	if set_type = 'dx' then output dx_score;
	if set_type = 'px' then output px_score;

	keep range_min range_max disease_number;
run;

**************************************;
* ICD9 dx format for frailty disease  ;
**************************************;
data master;
	set dx_score;

	label = left(disease_number);
	drop disease_number;
run;

data other;
	start = 'other';
	end   = 'other';
	label = 'other';
run;

data study_dx;
	set master(rename=(range_min = start range_max = end))
		other;

	fmtname = '$study_dx';
run;

**********************************************;
* cpt4 procedures format for frailty disease  ;
**********************************************;
data master;
	set px_score;

	label = left(disease_number);
	drop disease_number;
run;

data other;
	start = 'other';
	end   = 'other';
	label = 'other';
run;

data study_px;
	set master(rename=(range_min = start range_max = end))
		other;

	fmtname = '$study_px';
run;

proc format cntlin = study_px;
run;

data score_px;
	set dataout.px_&pyr.(keep = bene_id hcpcs_cd);

	class = put(hcpcs_cd, $study_px.);
	if class ^= 'other';

	last_d =  substr(left(hcpcs_cd), 5,1);
	if  last_d = ' '  or last_d = '0'  or last_d = '1'  or last_d ='2' or
		last_d = '3'  or last_d = '4'  or last_d = '5'  or last_d ='6' or
		last_d = '7'  or last_d = '8'  or last_d = '9';

	drop last_d;
run;

*****************************;
* frailty disease weight     ;
*****************************;
data weight;
	set r.disease_weight;

	class = left(disease_number);
	keep class weight;
run;

proc sort data = weight; by class; run;

proc sql;
	create table conditions_with_weight as
	select a.*, b.*
	from r.codes_org a inner join r.disease_weight b
	on a.disease_number=b.disease_number;
quit;

proc print data=conditions_with_weight; var description Disease_number weight ; run;
proc sort nodupkey data = score_px(keep=bene_id class); by bene_id class; run;
proc sort data = score_px; by class; run;

data scores_px;
	merge score_px(in = in1) weight(in = in2);
		by class;

	if in1 and in2;
run;

*******************************************;
* link icd10/disease/weight to             ;
* prior icd10 dx                           ;
*******************************************;
proc sql;
   create table score_dx10(keep = bene_id dx disease_number weight) as
   select * from r.frailty_icd10_dx diag, dataout.dx10_&pyr.  dx10
   where diag.dx = dx10.dx;
quit;

data score_10;
	set score_dx10;

	class  = left(disease_number);
	keep bene_id class weight;
run;

proc sort nodupkey data = score_10; by bene_id class; run;

data scores;
	set scores_px
		score_10;

	keep bene_id class weight;
run;

proc sort nodupkey data = scores; by bene_id class; run;

data count;
	set scores;
		by bene_id;

	if first.bene_id then score = 0.10288; /* intercept */
	score + weight;
	if last.bene_id then output;

	keep bene_id score;
run;

/* combine with full sample - anyone missing from claims above will be assigned intercept value = 0.10288 */
proc sql;
	create table frailty_score as
	select a.*, b.score
	from &denom_file. a left join count b
	on a.bene_id=b.bene_id;
quit;

/* this includes full denominator but will only be considered for those 65+ */
data dataout.frailty_score_&pyr._&yr.;
	set frailty_score;

	if score=. then score=0.10288; /* if a bene doesn't have any of the conditions, just set score to intercept */

	if age_&pyr. lt 65 then fscore_final=.;
	 else fscore_final=score;
run;

proc means data=dataout.frailty_score_&pyr._&yr. n nmiss mean median q1 q3 min max; var score; run;
proc means data=dataout.frailty_score_&pyr._&yr. n nmiss mean median q1 q3 min max; where age_&pyr. ge 65; var score; run;
